# -*- coding: utf-8 -*-
"""
Created on Tuesday January 14, 2020
Updated March 26 2023
Script allows to read and plott several FIT-files from local folder
Background is subtracted
x-axis is presented in DateTime-format
Edit: file, time for bckground subtraction in seconds
Edit: vmin and vmax for color table adjustment and edit color-table in imshow()
Decide for manualzoom. If True, set hour, minute and seconds appropriate for start and stop
and edit frequency range
Edit graphic-file extension if required png, pdf, jpg, ....
Edit rotation angle fir x- and y-axis if required, depends on duration of plot

@author: Christian Monstein
"""
#-------------------------------------------------------------------------------

from astropy.io import fits
import matplotlib.pyplot as plt 
from matplotlib import cm
import numpy as np
import matplotlib.dates as mdates
import datetime as dt
import time
import glob
from scipy.signal import savgol_filter

#-------------------------------------------------------------------------------
# User input parameter

MyPath   = '' # Edit in case the file is elsewhere, outside local folder, use '/' instead of '\'
MyFile1  = 'SSRT*_59.fit.gz' # filename including wildcards ? and *
Farray   = [248.25, 249.813, 252.563, 256.688, 262.938] # table of interesting frequencies
vmin     = -2 # minimum colour table value
vmax     = 17 # maximum colour table value
numberofxticks = 9 # to make 'nice' time lables between zoomTstart and zoomTstop 

manualzoom = 'True'
if (manualzoom):
    zoomTstart = [ 0, 0, 0] # start time as hour, minute, second. No leading zero!
    zoomTstop  = [ 1, 0, 0] # stop time as hour, minute, second. No leading zero!
    zoomF = [55,440] # frequency range min, max [MHz] for spectrum plot. Float o.k.

Tmin =  1 # Start time for background averaging [sec]
Tmax = 10 # Stop time for background averaging [sec]
    
#------------------------------------------------------------------------------

paths = glob.glob(MyPath + MyFile1) # search the folder
paths.sort() # force alphabetical order of the files
print ('\nFIT-files found: ',(len(paths)),paths)

#-------------------------------------------------------------------------------

def get_data_from_fits(path):
    with fits.open(path) as hdu:
        data = hdu[0].data.astype(np.uint8) # float16, float32, uin8 depends on PC
    return data

#-------------------------------------------------------------------------------

with fits.open(paths[0]) as hdu:
    data       = hdu[0].data
    freqs      = hdu[1].data  ['Frequency'][0] # extract frequency axis
    timeax     = hdu[1].data  ['Time'][0]      # extract time axis
    dT         = hdu[0].header['CDELT1']     # extract time resolution
    datum      = hdu[0].header['DATE-OBS']   # take first file
    T0         = hdu[0].header['TIME-OBS']   # take first file
    instrument = hdu[0].header['INSTRUME']
    content    = hdu[0].header['CONTENT']
    rows       = hdu[0].header['NAXIS2']  # constant over all FIT-files
    cols       = hdu[0].header['NAXIS1'] # may change per FIT-file ...!

    # print (hdu.info())
    # print (hdu[0].header)
    # print (hdu[1].header)
    
with fits.open(paths[-1]) as hdu:
    T1 = hdu[0].header['TIME-END']  # take first file only

#------------------------------------------------------------------------------

data = np.hstack([get_data_from_fits(path) for path in paths])
print ('Data shape: ',data.shape)

start_time = float(fits.open(paths[0])[0].header['TIME-OBS'].split(":")[0])*60*60 \
           + float(fits.open(paths[0])[0].header['TIME-OBS'].split(":")[1])*60 \
           + float(fits.open(paths[0])[0].header['TIME-OBS'].split(":")[2])  

time_axis = (start_time + dT * np.arange(data.shape[1]))/3600.0

#------------------------------------------------------------------------------

dmean = np.mean(data[:,int(Tmin/dT):int(Tmax/dT)], axis=1,keepdims=True) # all frequencies, time range in pixels
bgs1 = data - dmean # subtraction background

#-------------------------------------------------------------------------------
# Generate spectrum plot with axes

YY = int(datum.split("/")[0])
MM = int(datum.split("/")[1])
DD = int(datum.split("/")[2])

hh = int(T0.split(":")[0])
mm = int(T0.split(":")[1])
ss = int(0.5+float(T0.split(":")[2]))
if (ss>59):
    ss=59
d0 = dt.datetime(YY,MM,DD,hh,mm,ss)
unixtime0 = time.mktime(d0.timetuple())

hh = int(T1.split(":")[0])
mm = int(T1.split(":")[1])
ss = int(0.5+float(T1.split(":")[2]))
d1 = dt.datetime(YY,MM,DD,hh,mm,ss)
unixtime1 = time.mktime(d1.timetuple())

# Create your x-limits. Using two of your unix timestamps you first
# create a list of datetime.datetime objects using map.
x_lims = list(map(dt.datetime.fromtimestamp, [unixtime0, unixtime1]))

# You can then convert these datetime.datetime objects to the correct
# format for matplotlib to work with.
x_lims = mdates.date2num(x_lims)

# Set some initial y-limits.
y_lims = [freqs[-1],freqs[0]]

fig, ax = plt.subplots(figsize=(10,6.2))

# Using ax.imshow we set two keyword arguments. The first is extent.
# We give extent the values from x_lims and y_lims above.
# We also set the aspect to "auto" which should set the plot up nicely.
ax.imshow(bgs1, extent = [x_lims[0], x_lims[1],  y_lims[0], y_lims[1]], 
          aspect='auto',cmap=cm.gnuplot2,norm=plt.Normalize(vmin, vmax))
# https://matplotlib.org/examples/color/colormaps_reference.html
 # jet, CMRmap, gnuplot2, magma, plasma, Spectral, bone_r
# polarisation: Spectral, sonst jet

# We tell Matplotlib that the x-axis is filled with datetime data, 
# this converts it from a float (which is the output of date2num) 
# into a nice datetime string.
ax.xaxis_date()

ax.set_xlabel('Observation time [UTC]',fontsize=14)
ax.set_ylabel('Frequency [MHz]',fontsize=14)
#ax.set_title('Callisto: ' + MyFile,fontsize=15)
ax.set_title(content,fontsize=15)

for tick in ax.get_xticklabels():
    tick.set_rotation(0) # 20° ... 45°
    tick.set_size(12)
    
for tick in ax.get_yticklabels():
    tick.set_rotation(0) # 0° or 90°
    tick.set_size(12)

if (manualzoom=='True'):
    ax.set_ylim(zoomF)
    
    d2 = dt.datetime(YY,MM,DD,zoomTstart[0],zoomTstart[1],zoomTstart[2])
    d3 = dt.datetime(YY,MM,DD,zoomTstop[0], zoomTstop[1], zoomTstop[2])
    unixtime2 = time.mktime(d2.timetuple())
    unixtime3 = time.mktime(d3.timetuple())
    x_lims_new = list(map(dt.datetime.fromtimestamp, [unixtime2, unixtime3]))
    
    ax.set_xlim(x_lims_new)
    
    from matplotlib.ticker import (AutoMinorLocator)
    ax.xaxis.set_minor_locator(AutoMinorLocator())
    
    ax.tick_params(which='both' , width =2)
    ax.tick_params(which='major', length=5)
    ax.tick_params(which='minor', length=2, color='blue')

from matplotlib.dates import DateFormatter

fig.gca().xaxis.set_major_formatter( DateFormatter('%H:%M:%S') )

start, end = ax.get_xlim()
deltaT = (end-start)/numberofxticks
ax.xaxis.set_ticks(np.arange(start, end, deltaT))

fig.tight_layout() # makes x-axis label visible
plt.savefig("DynamicSpectrum_%s.png"%(datum.replace("/", "_")),bbox_inches="tight")

#-------------------------------------------------------------------------------
# Generate average spectrum

plt.figure()
yy = np.median(data,axis=1)
plt.step(freqs, yy,'g')
plt.xlabel("Frequency [MHz]",size=14)
plt.ylabel("Intensity [digit]",size=14)
plt.title("Averaged spectrum to be used for background subtraction.")
plt.axis((zoomF[0],zoomF[1],np.min(yy),np.max(yy))) # UT/MHz
plt.grid()
plt.tick_params(labelsize=12)
plt.savefig("Averagespectrum_%s.png"%(datum.replace("/", "_")),bbox_inches="tight")

#-------------------------------------------------------------------------------
# link channel numbers with frequencies and plot light-curves

Carray = []

for f in Farray:
    for ch in range(0,rows):
        if (freqs[ch] <= f):
            ccc = ch
            break
    print ('Channel index:',ccc,'-> Frequency:',f,'MHz')
    Carray.append(ccc)

plt.figure()
j = 0 # count channel
for channel in Carray:
    y = bgs1[channel,:]
    yf = savgol_filter(y, 21, 3)
    st= '{:5.1f} MHz'.format(Farray[j])
    j = j + 1
    plt.plot(time_axis,yf, label=st)
    
plt.xlabel("Time [UT]",fontsize=14)
plt.ylabel("Linear relative power",fontsize=14)
plt.title(instrument + ' lightcurve at fixed frequency channels',fontsize=14)
plt.tick_params(labelsize=12)
plt.legend(loc = "upper left")
plt.savefig("LightCurve_%s.png"%(datum.replace("/", "_")),bbox_inches="tight")

plt.show()

#-------------------------------------------------------------------------------
# Calculation s4 and plots

plt.figure()
plt.ylim(0,1.0)
plt.xlabel("Time [UT]",fontsize=14)
plt.ylabel("S4",fontsize=14)
plt.title(instrument + ' scintillation coefficients',fontsize=14)
plt.tick_params(labelsize=12)
plt.grid('both')

Tint = 60.0 # integration time [s] to get s4
Tobs = (time_axis[-1] - time_axis[0]) * 3600 # total observation time [s]
N    = int(Tint/dT) # number of slots
M    = int(Tobs/Tint) # number of pixels per slot to evaluate

j = 0
for channel in Carray:
    digits = bgs1[channel,:]
    dB = digits/255*2500/25.4
    lcLin = 10.0**(dB/10)
    s4  = []
    xs4 = []
    for i in range(0, N):
        Ipart = lcLin[i*M:(i+1)*M]
        Iz = np.mean(Ipart**2)
        In = np.mean(Ipart)**2
        s4.append(np.sqrt(Iz/In-1.0))
        xs4.append((i*M*dT+start_time)/3600.0)
    
    st= '{:5.1f} MHz'.format(Farray[j])
    j = j + 1
    plt.plot(xs4,s4, label=st)
    
plt.legend(loc = "upper left")
    
plt.savefig("S4_%s.png"%(datum.replace("/", "_")),bbox_inches="tight")
plt.show()

#-------------------------------------------------------------------------------

